@@ -6,7 +6,6 @@
 */
 
 #include "DeclarativeDragArea.h"
-#include "DeclarativeMimeData.h"
 
 #include <QDrag>
 #include <QGuiApplication>
@@ -195,7 +194,7 @@ void DeclarativeDragArea::setDefaultAction(Qt::DropAction action)
 void DeclarativeDragArea::mousePressEvent(QMouseEvent *event)
 {
     m_pressAndHoldTimerId = startTimer(QGuiApplication::styleHints()->mousePressAndHoldInterval());
-    m_buttonDownPos = event->screenPos();
+    m_buttonDownPos = event->globalPosition();
     m_draggingJustStarted = true;
     setKeepMouseGrab(true);
 }
@@ -236,7 +235,7 @@ void DeclarativeDragArea::timerEvent(QTimerEvent *event)
 
 void DeclarativeDragArea::mouseMoveEvent(QMouseEvent *event)
 {
-    if (!m_enabled || QLineF(event->screenPos(), m_buttonDownPos).length() < m_startDragDistance) {
+    if (!m_enabled || QLineF(event->globalPosition(), m_buttonDownPos).length() < m_startDragDistance) {
         return;
     }
 
@@ -279,18 +278,24 @@ bool DeclarativeDragArea::childMouseEventFilter(QQuickItem *item, QEvent *event)
     switch (event->type()) {
     case QEvent::MouseButtonPress: {
         QMouseEvent *me = static_cast<QMouseEvent *>(event);
-        // qDebug() << "press in dragarea";
-        mousePressEvent(me);
+        if (me->buttons() & acceptedMouseButtons()) {
+            // qDebug() << "press in dragarea";
+            mousePressEvent(me);
+        }
         break;
     }
     case QEvent::MouseMove: {
         QMouseEvent *me = static_cast<QMouseEvent *>(event);
-        // qDebug() << "move in dragarea";
-        mouseMoveEvent(me);
+        if (me->buttons() & acceptedMouseButtons()) {
+            // qDebug() << "move in dragarea";
+            mouseMoveEvent(me);
+        }
         break;
     }
     case QEvent::MouseButtonRelease: {
         QMouseEvent *me = static_cast<QMouseEvent *>(event);
+        // This event must accept all mouse buttons, because when the
+        // mouse is released me->buttons() will be Qt::NoButton.
         // qDebug() << "release in dragarea";
         mouseReleaseEvent(me);
         break;
@@ -367,3 +372,5 @@ void DeclarativeDragArea::startDrag(const QImage &image)
 
     ungrabMouse();
 }
+
+#include "moc_DeclarativeDragArea.cpp"
